/*******************************************************************************

File:   AlimFunc.h
Author : Luca Tanganelli (shinshi)
Version : 1.0

Created on Date : 31/03/2013
Last update     : 10/08/2013

CopyRight 2013 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Luca Tanganelli (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


********************************************************
PURPOSES
********************************************************


 Libreria di Funzioni dedicate per L'Alimentatore digitale da banco

*******************************************************************************/

#ifndef ALIMFUNC_H
#define	ALIMFUNC_H

#include <ctype.h>

/**
 * Funzione di arrotondamento.
 *
 * @param value valore int da arrotondare
 *
 * @param shift numero di scorrimenti a destra delle cifre
 *
 * @return valore arrotondato
 */
int rounding (int value, char shift);

/**
 * Converte un Integer in Stringa con virgola e lo visualizza sul display
 *  dipendente dalla libreria LCD_44780 di Mauro Laurenti.
 *
 * @param value  l'integer da convertire, non sono ammessi numeri negativi.
 *
 * @param number_of_digits Numero di caratteri che deve avere la stringa con giustificazione a destra.
 *                      deve essere maggiore di number_of_decimals
 *                      1-6 caratteri compresa la virgola
 *			es: 1,25 sono 4 caratteri
 *
 * @param number_of_decimals numero di caratteri dopo la virgola
 *
 * @note Non ammette i numeri negativi,
 *
 *
 */
void write_decimal_LCD(int value, char number_of_digits, char number_of_decimals);

/**
 * Funzione per la compensazione del valore del PWM se l'uscita  deversa dall'
 * impostazione del potenziometro.
 * Si rende necessario in quanto il valore in continua filtrato del segnale PWM
 * non  lineare.
 *
 * @param pot Valore impostato con il potenziometro.
 *
 * @param read Valore in uscita.
 *
 * @param comp valore attuale di compensazione.
 *
 * @return Valore compensazione.
 *
 *
 */
char offsetting(int pot, int read, char comp);

/**
 * Funzione per l'adattamento del segnale PWM con l'impostazione del potenziometro.
 *
 * @param raw_pot valore ADC del potenziometro.
 *
 * @param comp valore di compensazione.
 *
 * @return valore da inserire nel PWM.
 *
 *
 */
int pot2pwm (int raw_pot, char comp);

#endif	/* ALIMFUNC_H */
