/*******************************************************************************

File:   AlimHardlib.h
Author : Luca Tanganelli (shinshi)
Version : 1.0

Created on Date : 31/03/2013
Last update     : 10/08/2013

CopyRight 2013 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Luca Tanganelli (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*******************************************************************************/

#include "AlimHardlib.h"

//******************************************************************************
//                  FUNZIONI
//******************************************************************************
//******************************************************************************
//                  INIZIALIZZAZIONE
//******************************************************************************
void initialization (void) {

//******************************************************************************
//                  PORT Initialization
//******************************************************************************
/*Imposto PORTA
PIN     FUNCTION
2	AN0: Analog input 0	ANALOG IN	(READVOLT) input Tensione
3	AN1: Analog input 1	ANALOG IN	(READAMP) input Corrente
4	AN2: Analog input 2	ANALOG IN	(POTVOLT)  potenziomento VOLT
5	AN3: Analog input3	ANALOG IN	(POTAMP)  potenziometro AMPERE
6	RA4: Digital I/O	DIGITAL IN	NC
7	AN4: Analog input 4	ANALOG IN	(TEMP)  sonda temperatura
*/
    LATA = 0x00;
    TRISA = 0xFF;

/*Imposto PORTB
PIN     FUNCTION
33	RB0: Digital I/O	DIGITAL IN	NC
34	RB1: Digital I/O	DIGITAL IN	NC
35	RB2: Digital I/O	DIGITAL IN	NC
36	RB3: Digital I/O	DIGITAL IN	NC
37	RB4: Digital I/O	DIGITAL IN	(S405) Pulsante
38	RB5: Digital I/O	DIGITAL IN	(S404) Pulsante
39	RB6: Digital I/O	DIGITAL IN	(S403) Pulsante
40	RB7: Digital I/O	DIGITAL IN	(S402) Pulsante
*/
    LATB = 0x00;
    TRISB = 0xFF;

/*Imposto PORTC
PIN     FUNCTION
15	RC0: Digital I/O	DIGITAL OUT	(BUZZ) uscita buzzer
16	CCP2: PWM2 output	ANALOG OUT	(AMP) PWM out regolazione limitazione Corrente
17	CCP1: PWM1 output	ANALOG OUT	(VOLT) PWM out regolazione uscita Tensione
18	VUSB: Internal USB 3.3V voltage regulator output, positive supply for the USB transceiver.		Condensatore 470nF
23	D-: USB minus line 	DIGITAL IN/OUT	(D-) USB
24	D+: USB  plus line 	DIGITAL IN/OUT	(D+) USB
25	RC6: Digital I/O	DIGITAL IN	USB Detect
26	RC7: Digital I/O	DIGITAL OUT	(FAN) Accensione ventola
*/
    LATC = 0x00;
    TRISC = 0b01110000;

/*Imposto PORTD
PIN     FUNCTION
19	RD0: Digital I/O	DIGITAL IN	NC
20	RD1: Digital I/O	DIGITAL OUT	(RD1) Display R/W
21	RD2: Digital I/O	DIGITAL OUT	(RD2) Display RS
22	RD3: Digital I/O	DIGITAL OUT	(RD3) Display E
27	RD4: Digital I/O	DIGITAL OUT	(RD4) Display D4
28	RD5: Digital I/O	DIGITAL OUT	(RD5) Display D5
29	RD6: Digital I/O	DIGITAL OUT	(RD6) Display D6
30	RD7: Digital I/O	DIGITAL OUT	(RD7) Display D7
*/
    LATD = 0x00;
    TRISD = 0b00000001;

/*Imposto PORTE
PIN     FUNCTION
8	AN5: Analog input 5	ANALOG IN	NC
9	RE1: Digital I/O	DIGITAL OUT	(ATTIVA) Rel attivazione uscita
10	RE2: Digital I/O	DIGITAL OUT	(SCARICA) Rel per la funzione di scaricabatterie
*/
    LATE = 0x00;
    TRISE = 0x11111001;

//******************************************************************************
//                  Inizializzazione ADC
//******************************************************************************

    // Abilito AN0-AN5 come ingressi analogici
    ADCON1bits.PCFG = 0b1001;
    // VREF sono impostate a massa e VCC
    ADCON1bits.VCFG = 0b00;
    //Tempo di acquisizione 12TAD*16/20Mhz = 9,6us
    // TAD : FOSC/16 = 0b101
    ADCON2bits.ADCS = 0b101;
    // TACQ: 12 TAD = 0b101
    ADCON2bits.ACQT = 0b101;
    //Giustificazione a destra
    ADCON2bits.ADFM = 1;
    // Abilito l' ADC
    ADCON0bits.ADON = 1;

//******************************************************************************
//                  Inizializzazione PWM
//******************************************************************************

    // Apro il TMR2 per il PWM
    OpenTimer2 (TIMER_INT_OFF & T2_PS_1_1 & T2_POST_1_1);
    // Apro i molduli PWM
    OpenPWM1(PERIOD);
    OpenPWM2(PERIOD);

}

//******************************************************************************
//                  LETTURA ADC
//******************************************************************************
int ADC_read (char channel, char redundancy){

    // Variabile per salvare la sommatoria dei dati letti
    int sommatoria = 0;
    // Variabile per salvare il valore della conversione
    int lettura = 0;
    // Variabile utilizzata per il numero delle letture
    char i;

    // Seleziono ingresso
    ADCON0bits.CHS = channel;

    // Effettuo tot letture per fare una media
    for (i =0; i<(1 << redundancy); i++){
        // Avvio la conversione Analogico/Digitale
        ADCON0bits.GO = 1;
        // Attendo la fine della conversione
        while(ADCON0bits.GO);
        // Prelevo il valore della conversione
        lettura = (((int) ADRESH) << 8) | ADRESL;
        // Sommatoria delle letture fatte
        sommatoria = sommatoria + lettura;
    }
    // shift per la media
    sommatoria = sommatoria >> redundancy;
    return sommatoria;

}


//******************************************************************************
//                  GESTIONE VENTOLA
//******************************************************************************
void tempcheck (void){
    unsigned char temperature;
    temperature = (ADC_read (TEMP, 1)) >> 1;

	if (temperature > TEMP_MAX){
            FAN = TURN_ON;
            ATTIVA = TURN_OFF;
            SetDCPWM1 (0);
            SetDCPWM2 (0);
            while (1){
                BUZZ = TURN_ON;
            }
	}

	if (temperature > TEMP_WARNING){
		FAN = TURN_ON;
	} else
 	if (temperature < TEMP_WARNING - TEMP_HYSTERESIS)
		FAN = TURN_OFF;

}

